@@ -14,6 +14,7 @@ import shutil
 import subprocess
 import sys
 import textwrap
+import webbrowser
 import zipfile
 from pathlib import Path
 from pipes import quote
@@ -27,13 +28,13 @@ from cloudmesh.common.dotdict import dotdict
 from cloudmesh.common.systeminfo import get_platform
 from cloudmesh.common.util import path_expand
 from cloudmesh.common.util import readfile
+from cloudmesh.common.util import writefile
 from cloudmesh.common.systeminfo import os_is_windows
 from cloudmesh.common.systeminfo import os_is_mac
 from cloudmesh.common.systeminfo import os_is_linux
 from cloudmesh.common.util import is_gitbash
 
 
-
 # from functools import wraps
 # def timer(func):
 #    @wraps(func)
@@ -65,6 +66,7 @@ def windows_not_supported(f):
 
     return wrapper
 
+
 def NotImplementedInWindows():
     if sys.platform == "win32":
         Console.error(f"The method {__name__} is not implemented in Windows.")
@@ -244,6 +246,23 @@ class Shell(object):
             Console.warning('Could not determine locale. Defaulting to "us"')
             return 'us'
 
+    @staticmethod
+    def ssh_enabled():
+        if os_is_linux():
+            r = Shell.run("service sshd status | fgrep running").strip()
+            return len(r) > 0
+        elif os_is_windows():
+            r = Shell.run("ps | grep -F ssh")
+            return "ssh" in r
+        elif os_is_mac():
+            r = Shell.run("ps -ef")
+            if "sshd" in r:
+                print('IT WORKS!!!!!')
+            else:
+                print('it doenst work :(')
+            return "sshd" in r
+        return False
+
     @staticmethod
     def run_timed(label, command, encoding=None, service=None):
         """
@@ -262,7 +281,7 @@ class Shell(object):
         return str(result)
 
     @staticmethod
-    def run(command, exit="; exit 0", encoding='utf-8', replace=True):
+    def run(command, exit="; exit 0", encoding='utf-8', replace=True, timeout=None):
         """
         executes the command and returns the output as string
         :param command:
@@ -275,13 +294,19 @@ class Shell(object):
                 c = "&"
             else:
                 c = ";"
-            command = f"{command}".replace(";",c)
+            command = f"{command}".replace(";", c)
         else:
             command = f"{command} {exit}"
 
-        r = subprocess.check_output(command,
-                                    stderr=subprocess.STDOUT,
-                                    shell=True)
+        if timeout is not None:
+            r = subprocess.check_output(command,
+                                        stderr=subprocess.STDOUT,
+                                        shell=True,
+                                        timeout=timeout)
+        else:
+            r = subprocess.check_output(command,
+                                        stderr=subprocess.STDOUT,
+                                        shell=True)
         if encoding is None or encoding == 'utf-8':
             return str(r, 'utf-8')
         else:
@@ -413,7 +438,7 @@ class Shell(object):
                     # shell=True,
                     stderr=subprocess.STDOUT,
                     cwd=cwd)
-        except:
+        except:  # noqa: E722
             if witherror:
                 Console.error("problem executing subprocess",
                               traceflag=traceflag)
@@ -477,22 +502,112 @@ class Shell(object):
         os.system(command)
 
     @staticmethod
-    def browser(filename=None, engine='python -m webbrowser -t'):
-        if ".svg" in filename:
-            if os_is_linux():
-                if engine.startswith("chrome"):
-                    os.system(f"chromium {filename}")
-                else:
-                    os.system(f"gopen {filename}")
-            elif os_is_mac():
-                os.system(f"open {filename}")
-            elif os_is_windows():
-                raise NotImplementedError("IMplement me in Shell.browser")
-        else:
-            if 'file:' not in filename and 'http' not in filename:
-                os.system(f"{engine} file:///{filename}")
+    def map_filename(name):
+        pwd = os.getcwd()
+
+        _name = str(name)
+        result = dotdict()
+
+        if os_is_windows():
+            if "USERNAME" in os.environ:
+                result.user = os.path.basename(os.environ["USERNAME"])
             else:
-                os.system(f"{engine} {file}".format(**data))
+                Console.error("Could not detect the home directory")
+        else:
+            result.user = os.path.basename(os.environ["HOME"])
+        result.host = "localhost"
+        result.protocol = "localhost"
+
+        if _name == "":
+            result.path = ""
+            if result.host == "localhost":
+                _name = "."
+
+        if _name.startswith("http"):
+            result.path = _name
+            result.protocol = _name.split(':', 1)[0]
+        elif _name.startswith("wsl:"):
+            result.path = _name.replace("wsl:", "")
+            # Abbreviations: replace ~ with home dir and ./ + / with pwd
+            if result.path.startswith("~"):
+                if os_is_linux():
+                    result.path = result.path.replace("~", f"/mnt/c/home/{result.user}")
+                else:
+                    result.path = result.path.replace("~", f"/mnt/c/Users/{result.user}")
+            elif not result.path.startswith("/"):
+                if os_is_windows():
+                    pwd = pwd.replace("C:", "/mnt/c").replace("\\", "/")
+                result.path = pwd + "/" + result.path.replace("./", "")
+            result.protocol = "cp"
+            result.host = "wsl"
+        elif _name.startswith("scp:"):
+            # scp source destination
+            # command = f"scp:username@host:hi.txt"
+            try:
+                result.scp, userhost, result.path = _name.split(":")
+                result.user, result.host = userhost.split("@")
+                result.protocol = "scp"
+            except:  # noqa: E722
+                Console.error(f"The format of the name is not supported: {name}")
+        elif _name.startswith("rsync:"):
+            try:
+                result.rsync, userhost, result.path = _name.split(":")
+                result.user, result.host = userhost.split("@")
+                result.protocol = "rsync"
+            except:  # noqa: E722
+                Console.error(f"The format of the name is not supported: {name}")
+        elif _name.startswith("ftp:"):
+            # expecting the following format: ftp://user:password@myftpsite/myfolder/myfile.txt
+            try:
+                result.ftp_location, result.ftp_path = _name.split("@")
+                result.ftp_prefix, result.ftp_login = result.ftp_location.split("//")
+                result.username, result.password = result.ftp_login.split(":")
+                result.protocol = "ftp"
+            except:  # noqa: E722
+                Console.error(f"The format of the name is not supported: {name}")
+        elif _name.startswith(".") or _name.startswith("~"):
+            result.path = path_expand(_name)
+        elif _name[1] == ":":
+            drive, path = _name.split(":", 1)
+            if os_is_windows():
+                result.path = path_expand(path)
+            else:
+                result.path = drive + ":" + path_expand(path)
+            result.path = result.path.replace("/", "\\")
+        else:
+            result.path = path_expand(_name)
+
+        return result
+
+    @staticmethod
+    def browser(filename=None):
+        """
+        :param filename:
+        :return:
+        """
+        if not os.path.isabs(filename) and 'http' not in filename:
+            filename = Shell.map_filename(filename).path
+        webbrowser.open(filename, new=2)
+
+    @staticmethod
+    def fetch(filename=None, destination=None):
+        _filename = Shell.map_filename(filename)
+
+        content = None
+
+        if _filename.path.startswith('http'):
+            result = requests.get(_filename.path)
+            content = result.text
+
+        else:
+            os.path.exists(_filename.path)
+            content = readfile(_filename.path)
+
+        if destination is not None:
+            destination = Shell.map_filename(destination)
+            writefile(destination, content)
+
+        return content
 
     @staticmethod
     def terminal_title(name):
@@ -530,7 +645,7 @@ class Shell(object):
             )
         elif platform == "linux":  # for ubuntu running gnome
             dist = os_platform.linux_distribution()[0]
-            linux_apps = {'ubuntu': 'gnome-terminal', 'debian':'lxterminal'}
+            linux_apps = {'ubuntu': 'gnome-terminal', 'debian': 'lxterminal'}
             os.system(f"{linux_apps[dist]} -e \"bash -c \'{command}; exec $SHELL\'\"")
 
         elif platform == "win32":
@@ -540,7 +655,8 @@ class Shell(object):
             kind = kind.lower()
             if kind == "gitbash":
                 p = subprocess.Popen([r"C:\Program Files\Git\git-bash.exe",
-                                  "-c", f"{command}"])
+                                      "-c",
+                                      f"{command}"])
                 return p.pid
             elif kind == "cmd":
                 Console.error(f"Command not implemented for {kind}")
@@ -596,14 +712,17 @@ class Shell(object):
         return subprocess.check_output(*args, **kwargs)
 
     @classmethod
-    def ls(cls, match="."):
+    def ls(cls, directory=".", match=None):
         """
         executes ls with the given arguments
         :param args:
         :return: list
         """
-        d = glob.glob(match)
-        return d
+        if match == None:
+            files = os.listdir('.')
+        else:
+            files = [f for f in os.listdir('.') if re.match(match, f)]
+        return files
 
     @classmethod
     # @NotImplementedInWindows
@@ -616,7 +735,7 @@ class Shell(object):
         return cls.execute('ls', args)
 
     @staticmethod
-    def ps():
+    def ps(short=False, attributes=None):
         """
         using psutil to return the process information pid, name and comdline,
         cmdline may be a list
@@ -626,7 +745,14 @@ class Shell(object):
         found = []
         for proc in psutil.process_iter():
             try:
-                pinfo = proc.as_dict(attrs=['pid', 'name', 'cmdline'])
+                if attributes is not None:
+                    pinfo = proc.as_dict(attrs=attributes)
+                elif short:
+                    pinfo = proc.as_dict(attrs=['pid', 'name', 'cmdline', 'ppid', 'username',
+                                                'status', 'create_time', 'terminal', 'cwd',
+                                                'open_files'])
+                else:
+                    pinfo = proc.as_dict()
             except psutil.NoSuchProcess:
                 pass
             else:
@@ -731,17 +857,15 @@ class Shell(object):
         return cls.execute('cmsd', args)
 
     @classmethod
-    # @NotImplementedInWindows
-    def head(cls, *args):
+    def head(cls, filename=None, lines=10):
         """
         executes head with the given arguments
         :param args:
         :return:
         """
-        if not is_gitbash():
-            content = Shell.cat(args[0]).splitlines()
-            return content[0]
-        return cls.execute('head', args)
+        filename = cls.map_filename(filename).path
+        r = Shell.run(f'head -n {lines} {filename}')
+        return r
 
     @classmethod
     def keystone(cls, *args):
@@ -811,7 +935,7 @@ class Shell(object):
 
         with open(destination, 'wb') as fd:
             with tqdm(total=total_size, unit="B",
-               unit_scale=True, desc=destination, initial=0, ascii=True) as pbar:
+                      unit_scale=True, desc=destination, initial=0, ascii=True) as pbar:
                 for chunk in r.iter_content(chunk_size=chunk_size):
                     fd.write(chunk)
                     pbar.update(len(chunk))
@@ -853,11 +977,15 @@ class Shell(object):
         :param count: the number of pings
         :return:
         """
-        option = '-n' if platform == 'windows' else '-c'
-        return cls.execute('ping',
-                           "{option} {count} {host}".format(option=option,
-                                                            count=count,
-                                                            host=host))
+        r = None
+        option = '-n' if os_is_windows() else '-c'
+        parameters = "{option} {count} {host}".format(option=option,
+                                                      count=count,
+                                                      host=host)
+        r = Shell.run(f'ping {parameters}')
+        if r is None:
+            Console.error("ping is not installed")
+        return r
 
     @classmethod
     def pwd(cls, *args):
@@ -878,15 +1006,17 @@ class Shell(object):
         return cls.execute('rackdiag', args)
 
     @classmethod
-    # @NotImplementedInWindows
     def rm(cls, location):
         """
-        executes rm tree with the given arguments
+        executes rm with the given arguments
         :param args:
         :return:
         """
-        shutil.rmtree(path_expand(location))
-
+        try:
+            location = cls.map_filename(location).path
+            os.remove(location)
+        except:
+            pass
     @classmethod
     def rsync(cls, *args):
         """
@@ -948,16 +1078,15 @@ class Shell(object):
         return cls.execute('sudo', args)
 
     @classmethod
-    # @NotImplementedInWindows
-    def tail(cls, *args):
+    def tail(cls, filename=None, lines=10):
         """
         executes tail with the given arguments
         :param args:
         :return:
         """
-        NotImplementedInWindows()
-        # TODO: implement with readlines on a file.
-        return cls.execute('tail', args)
+        filename = cls.map_filename(filename).path
+        r = Shell.run(f'tail -n {lines} {filename}')
+        return r
 
     @classmethod
     def vagrant(cls, *args):
@@ -1007,28 +1136,17 @@ class Shell(object):
         return cls.execute('pip', args)
 
     @classmethod
-    # @NotImplementedInWindows
-    def fgrep(cls, *args):
-        """
-        executes fgrep with the given arguments
-        :param args:
-        :return:
-        """
-        if os_is_windows() and not is_gitbash():
-            NotImplementedInWindows()
-        return cls.execute('fgrep', args)
+    def fgrep(cls, string=None, file=None):
+        if not os_is_windows():
+            r = Shell.run(f'fgrep {string} {file}')
+        else:
+            r = Shell.run(f'grep -F {string} {file}')
+        return r
 
     @classmethod
-    # @NotImplementedInWindows
-    def grep(cls, *args):
-        """
-        executes grep with the given arguments
-        :param args:
-        :return:
-        """
-        if os_is_windows() and not is_gitbash():
-            NotImplementedInWindows()
-        return cls.execute('grep', args)
+    def grep(cls, string=None, file=None):
+        r = Shell.run(f'grep {string} {file}')
+        return r
 
     @classmethod
     def cm_grep(cls, lines, what):
@@ -1254,10 +1372,39 @@ class Shell(object):
             s = path_expand(source)
             d = path_expand(destination)
         else:
-            s=source
+            s = source
             d = destination
         shutil.copy2(s, d)
 
+    @classmethod
+    def copy_file(cls, source, destination):
+
+        try:
+            s = Shell.map_filename(source)
+            d = Shell.map_filename(destination)
+
+            dest_dir = os.path.dirname(d.path)
+
+            Shell.mkdir(dest_dir)
+
+            if s.protocol in ['http', 'https']:
+                command = f'curl {s.path} -o {d.path}'
+                Shell.run(command)
+            elif s.protocol == 'scp':
+                Shell.run(f'scp {s.user}@{s.host}:{s.path} {d.path}')
+            elif s.protocol == 'ftp':
+                command = fr"curl -u {s.username}:{s.password} ftp://{s.ftp_path} -o {d.path}"
+                print(command)
+                Shell.run(command)
+            elif d.protocol == 'ftp':
+                command = fr"curl -T {s.path} ftp://{d.ftp_path} --user {d.username}:{d.password}"
+                print(command)
+                Shell.run(command)
+            else:
+                shutil.copy2(s.path, d.path)
+        except Exception as e:
+            Console.error(e, traceflag=True)
+
     @classmethod
     def mkdir(cls, directory):
         """
@@ -1265,23 +1412,22 @@ class Shell(object):
         :param directory: the path of the directory
         :return:
         """
-        directory = path_expand(directory)
-        Path(directory).mkdir(parents=True, exist_ok=True)
+        d = cls.map_filename(directory).path
+        try:
+            Path.mkdir(d, parents=True, exist_ok=True)
+            return True
+        except Exception as e:
+            if not os_is_windows():
+                command = f'mkdir -p {d}'
+            else:
+                command = f'mkdir {d}'
+            try:
+                os.system(command)
+                return True
+            except:
+                Console.error(e, traceflag=True)
+        return False
 
-        #try:
-        #    os.makedirs(directory)
-        #except OSError as e:
-
-            # EEXIST (errno 17) occurs under two conditions when the path exists:
-            # - it is a file
-            # - it is a directory
-            #
-            # if it is a file, this is a valid error, otherwise, all
-            # is fine.
-        #    if e.errno == errno.EEXIST and os.path.isdir(directory):
-        #        pass
-        #    else:
-        #        raise
 
     def unzip(cls, source_filename, dest_dir):
         """
@@ -1351,7 +1497,7 @@ class Shell(object):
                 elif "Ubuntu" in result["NAME"]:
                     result["distribution"] = "ubuntu"
 
-            except:
+            except:  # noqa: E722
                 try:
                     r = cls.lsb_release()
                     for line in r.split():
@@ -1363,7 +1509,7 @@ class Shell(object):
                             result[attribute] = value
                     result["distribution"] = result["description"].split(" ")[
                         0].lower()
-                except:
+                except:  # noqa: E722
                     Console.error(
                         f"lsb_release not found for the platform {machine}")
                     raise NotImplementedError
@@ -1379,6 +1525,40 @@ class Shell(object):
 
         return result
 
+    @staticmethod
+    def open(filename=None, program=None):
+        if not os.path.isabs(filename):
+            filename = path_expand(filename)
+
+        if os_is_linux():
+            r = os.system(f"gopen {filename}")
+        if os_is_mac():
+            command = f'open {filename}'
+            if program:
+                command += f' -a "{program}"'
+            r = Shell.run(command)
+        if os_is_windows():
+            r = Shell.run(f"start {filename}")
+
+        return r
+
+    @staticmethod
+    def sys_user():
+        if os_is_windows():
+            localuser = os.environ["USERNAME"]
+        else:
+            try:
+                localuser = os.environ['USER']
+            except:
+                # docker image does not have user variable. so just do whoami
+                localuser = Shell.run('whoami')
+        return localuser
+
+    @staticmethod
+    @property
+    def user():
+        return Shell.sys_user()
+
 
 def main():
     """
