@@ -89,18 +89,23 @@ We recommend tho install the newest version as follows
 
 ::
     git clone https://github.com/mlperf/logging.git mlperf-logging
-    pip install -e mlperf-logging
+    cd mlperf-logging
+    pip install -e .
 
 Now you can just use the STopwatch as before.
 
 We will add here aditional information, such as setting up the configuration for mlperf logging
 
+# TODO - Need to exercise this for mlperf_logging
+
 """
 import os
+import platform
 import time
 import datetime
 import pprint
-
+import pathlib
+import yaml
 import sys
 
 from cloudmesh.common.console import Console
@@ -112,6 +117,7 @@ from cloudmesh.common.DateTime import DateTime
 
 from time import perf_counter
 
+
 def rename(newname):
     """
     decorator to rename a function
@@ -145,6 +151,16 @@ def benchmark(func):
 
     return wrapper
 
+
+def import_mllog():
+    try:
+        from mlperf_logging import mllog
+    except:  # noqa: E722
+        Console.error("You need to install mllogging to use it")
+        sys.exit()
+    return mllog
+
+
 class StopWatch(object):
     """
     A class to measure times between events.
@@ -169,36 +185,137 @@ class StopWatch(object):
     mllogger = None
 
     @classmethod
-    def activate_mllog(cls, filename="cloudmesh_mllog.log"):
+    def activate_mllog(cls, filename="cloudmesh_mllog.log", config=None, stack_offset=2):
+        # global mllog
+        cls._mllog_import = import_mllog()
+
+        if config is None:
+            cms_mllog = dict(
+                default_namespace="cloudmesh",
+                default_stack_offset=stack_offset,
+                default_clear_line=False
+            )
+        else:
+            cms_mllog = config
+
+        cls.mllogging = True
+        cls.mllogger = cls._mllog_import.get_mllogger()
+        cls._mllog_import.config(filename=filename)
+        cls._mllog_import.config(**cms_mllog
+                                 # useful when refering to linenumbers in separate code
+                                 # root_dir=os.path.normpath(
+                                 #    os.path.join(os.path.dirname(os.path.realpath(__file__)), "..", ".."))
+                                 )
+
+    # @classmethod
+    # def progress(cls, percent, status="running", pid=None):
+    #     if pid is None:
+    #         pid = os.getpid()
+    #     if "SLURM_JOB_ID" in os.environ:
+    #         pid = os.environ["SLURM_JOB_ID"] #TODO - may need to be updated (monitor of long running jobs)
+    #     print(f"# cloudmesh status={status} progress={percent} pid={pid}")
+
+    @classmethod
+    def progress(cls, percent, status="running", pid=None, variable=None):
+        """Prints progress of an event, recording against a pid and providing additional variable.
+
+        :percent: 0-100 value
+        :status: Message to associate to the recording, default - running
+        :pid: The associated Process ID for this event.
+        :variable: Any valid python type with a __str__ method.
+
+        :returns: The progress message as a string
+        """
+        if pid is None:
+            pid = os.getpid()
+        if "SLURM_JOB_ID" in os.environ:
+            # TODO - may need to be updated (monitor of long running jobs)
+            pid = os.environ["SLURM_JOB_ID"]
+        msg = f"# cloudmesh status={status} progress={percent} pid={pid}"
+        if variable is not None:
+            msg = msg + f" variable={variable}"
+        print(msg)
+        return msg
+        try:
+            config = yaml.safe_load(readfile(configfile).strip())
+        except:  # noqa: E722
+            config = {
+                "benchmark": {}
+            }
+        config["benchmark"].update(argv)
+
+    # @classmethod
+    # def organization_mllog(cls, configfile = None, prefix_: str = 'benchmark', flatdict_: bool = False, **argv):
+    #     try:
+    #         from mlperf_logging import mllog
+    #     except Exception:  # noqa: E722
+    #         Console.error("You need to install mllogging to use it")
+    #         sys.exit()
+    #
+    #     try:
+    #         with open(pathlib.Path(configfile), 'r') as stream:
+    #             _config = yaml.safe_load(stream)
+    #     except Exception as e:  # noqa: E722
+    #         _config = {
+    #             "benchmark": {}
+    #         }
+    #     prefix = prefix_
+    #     if flatdict_:
+    #         for k,v in argv.items():
+    #             _config[f"{prefix}.{k}"] = v
+    #     else:
+    #         _config[prefix].update(argv)
+    #
+    #     for key, attribute in [
+    #         (mllog.constants.SUBMISSION_BENCHMARK, 'name'),
+    #         (mllog.constants.SUBMISSION_POC_NAME, 'user'),
+    #         (mllog.constants.SUBMISSION_POC_EMAIL, 'e-mail'),
+    #         (mllog.constants.SUBMISSION_ORG, 'organisation'),
+    #         (mllog.constants.SUBMISSION_DIVISION, 'division'),
+    #         (mllog.constants.SUBMISSION_STATUS, 'status'),
+    #         (mllog.constants.SUBMISSION_PLATFORM, 'platform')
+    #         ]:
+    #         try:
+    #             if flatdict_:
+    #                 cls.mllogger.event(key, value=_config[f"{prefix}.{attribute}"])
+    #             else:
+    #                 cls.mllogger.event(key, value=_config["benchmark"][attribute])
+    #         except AttributeError as e:
+    #             print(f"Missing/invalid standard property {key}")
+
+    @classmethod
+    def organization_mllog(cls, configfile=None, **argv):
         try:
             from mlperf_logging import mllog
-        except:
+        except Exception:  # noqa: E722
             Console.error("You need to install mllogging to use it")
             sys.exit()
 
-        cls.mllogging = True
-        cls.mllogger = mllog.get_mllogger()
-        mllog.config(filename=filename)
-        mllog.config(
-            default_namespace="cloudmesh",
-            default_stack_offset=1,
-            default_clear_line=False,
-            # useful when refering to linenumbers in separate code
-            # root_dir=os.path.normpath(
-            #    os.path.join(os.path.dirname(os.path.realpath(__file__)), "..", ".."))
-        )
+        try:
+            with open(pathlib.Path(configfile), 'r') as stream:
+                data = yaml.safe_load(stream)
+        except Exception as e:  # noqa: E722
+            data = {
+                "benchmark": {}
+            }
+            data.update(argv)
+        except Exception as e:
+            Console.error(e, traceflag=True)
 
-    @classmethod
-    def organization_mllog(cls, configfile, **argv):
-        import mllog
-        import yaml
-        config = yaml.safe_load(readfile(configfile).strip())
-        config["benchmark"].update(argv)
-        cls.mllogger.event(key=mllog.constants.SUBMISSION_BENCHMARK, value=config["benchmark"]['name'])
-        cls.mllogger.event(key=mllog.constants.SUBMISSION_ORG, value=config["benchmark"]['organisation'])
-        cls.mllogger.event(key=mllog.constants.SUBMISSION_DIVISION, value=config["benchmark"]['division'])
-        cls.mllogger.event(key=mllog.constants.SUBMISSION_STATUS, value=config["benchmark"]['status'])
-        cls.mllogger.event(key=mllog.constants.SUBMISSION_PLATFORM, value=config["benchmark"]['platform'])
+        for key, attribute in [
+            (mllog.constants.SUBMISSION_BENCHMARK, 'name'),
+            (mllog.constants.SUBMISSION_POC_NAME, 'user'),
+            (mllog.constants.SUBMISSION_POC_EMAIL, 'e-mail'),
+            (mllog.constants.SUBMISSION_ORG, 'organisation'),
+            (mllog.constants.SUBMISSION_DIVISION, 'division'),
+            (mllog.constants.SUBMISSION_STATUS, 'status'),
+            (mllog.constants.SUBMISSION_PLATFORM, 'platform')
+        ]:
+            try:
+                value = data["benchmark"][attribute]
+                cls.event(key, mllog_key=key, value=value, stack_offset=3, suppress_stopwatch=True)
+            except Exception as e:
+                Console.error(e, traceflag=True)
 
 
     @classmethod
@@ -246,78 +363,204 @@ class StopWatch(object):
         cls.timer_msg[name] = value
 
     @classmethod
-    def event(cls, name, msg=None, values=None):
+    def event(cls, name, msg=None, values=None, value=None, mllog_key=None, suppress_stopwatch=False, suppress_mllog=False,
+              stack_offset=2):
         """
-        adds an event with a given name, where start and stop is the same tme.
+        Adds an event with a given name, where start and stop is the same time.
 
         :param name: the name of the timer
         :type name: string
-        :param data: data that is associated with the event that is converted to a string
-        :type data: object
-        """
-        StopWatch.start(name)
-        StopWatch.stop(name)
-        StopWatch.timer_end[name] = StopWatch.timer_start[name]
-        if values:
-            StopWatch.timer_values[name] = values
+        :param msg: a message to attach to this event
+        :type msg: string
+        :param values: data that is associated with the event that is converted
+                       to a string
+        :type values: object
+        :param mllog_key: Specifies the key to be used in mlperf_logging.
+                          If none, it will use the `name` value.
+        :type mllog_key: string
+        :param suppress_stopwatch: suppresses executing any stopwatch code.
+                                Useful when only logging an mllog event.
+        :type suppress_stopwatch: bool
+        :param suppress_mllog: suppresses executing any mllog code.  Useful
+                               when only interacting with stopwatch timers.
+        :type suppress_mllog: bool
 
-        if msg is not None:
-            StopWatch.message(name, str(msg))
-        if cls.mllogging:
-            if values is not None:
-                cls.mllogger.event(key=f"mllog-event-{name}", value=str(values))
+        :returns: None
+        :rtype: None
+        """
+        values = values or value
+        if not suppress_stopwatch:
+            StopWatch.start(name, suppress_mllog=True)
+            StopWatch.stop(name, suppress_mllog=True)
+            StopWatch.timer_end[name] = StopWatch.timer_start[name]
+            if values:
+                StopWatch.timer_values[name] = values
+
+            if msg is not None:
+                StopWatch.message(name, str(msg))
+        if cls.mllogging and not suppress_mllog:
+            if mllog_key is None:
+                key_name = cls._mllog_lookup("POINT_IN_TIME")
             else:
-                cls.mllogger.event(key=f"mllog-event-{name}")
+                key_name = cls._mllog_lookup(mllog_key)
+
+            if values is not None:
+                cls.mllogger.event(key=name, value=str(values), stack_offset=stack_offset)
+            else:
+                cls.mllogger.event(key=name, stack_offset=stack_offset)
 
     @classmethod
-    def start(cls, name, values=None):
+    def log_event(cls, **kwargs):
+        """Logs an event using the passed keywords as parameters to be logged,
+           prefiltered by mlperf_logging's standard api.
+
+        :param kwargs: an unpacked dictionary of key=value entries to be
+                       leveraged when logging an event to both the cloudmesh
+                       stopwatch and mllog.  If the keyword matches
+                       mlperf_logging's constants, the value will be replaced
+                       with the standardized string
+        :type kwargs: dict
+        """
+        for key, value in kwargs.items():
+            mlkey = cls._mllog_lookup(key)
+            cls.event(mlkey, msg=mlkey, values=value, stack_offset=3)
+
+    @classmethod
+    def _mllog_lookup(cls, key: str) -> str:
+        """Performs a dynamic lookup for the string representation of a
+           mlperf constant.  If the value isn't found, it will return a string
+           of the pattern mllog-event-{key}
+
+        :param key: The name of the constant to look up
+        :type key: string
+
+        :returns: The decoded value of the constant.
+        :rtype: string
+        """
+        try:
+            from mlperf_logging.mllog import constants as mlconst
+        except ImportError as e:
+            Console.error("You need to install mlperf_logging to use it")
+            raise e
+        try:
+            key_str = getattr(mlconst, key)
+        except AttributeError as e:
+            key_str = f"mllog-event-{key}"
+        return key_str
+
+    @classmethod
+    def start(cls, name, values=None, value=None, mllog_key=None, suppress_stopwatch=False, suppress_mllog=False,
+              metadata=None):
         """
         starts a timer with the given name.
 
         :param name: the name of the timer
         :type name: string
-        """
-        if cls.debug:
-            print("Timer", name, "started ...")
-        if name not in cls.timer_sum:
-            cls.timer_sum[name] = 0.0
-        cls.timer_start[name] = time.time()
-        cls.timer_end[name] = None
-        cls.timer_status[name] = None
-        cls.timer_msg[name] = None
-        if values:
-            StopWatch.timer_values[name] = values
+        :param values: any python object with a __str__ method to record with
+                       the event.
+        :type values: object
+        :param mllog_key: Specifies the string name of an mllog constant to
+                          associate to this timer start.  If no value is passed
+                          and mllogging is enabled, then `name` is used.
+        :type mllog_key: string
+        :param suppress_stopwatch: When true, prevents all traditional
+                                   stopwatch logic from running.  This is
+                                   useful when attempting to interact with
+                                   mllog-only.
+        :type suppress_stopwatch: bool
+        :param suppress_mllog: When true, prevents all mllog events from
+                               executing.  Useful when working with stopwatch
+                               timers-only.
 
-        if cls.mllogging:
-            if values is not None:
-                cls.mllogger.start(key=f"mllog-timer-{name}", value=str(values))
+        :returns: None
+        :rtype: None
+        """
+        values = values or value
+
+        if not suppress_stopwatch:
+            if cls.debug:
+                print("Timer", name, "started ...")
+            if name not in cls.timer_sum:
+                cls.timer_sum[name] = 0.0
+            cls.timer_start[name] = time.time()
+            cls.timer_end[name] = None
+            cls.timer_status[name] = None
+            cls.timer_msg[name] = None
+            if values:
+                StopWatch.timer_values[name] = values
+
+        if cls.mllogging and not suppress_mllog:
+            if mllog_key is None:
+                key = name
             else:
-                cls.mllogger.start(key=f"mllog-timer-{name}")
+                key = cls._mllog_lookup(mllog_key)
+            if values is not None:
+                if isinstance(values, dict):
+                    values['name'] = name
+                elif isinstance(values, list):
+                    values += list(name)
+                else:
+                    values = f"Name: {name}, {values}"
+
+                cls.mllogger.start(key=key, value=str(values), metadata=metadata)
+            else:
+                cls.mllogger.start(key=key, value=name, metadata=metadata)
 
     @classmethod
-    def stop(cls, name, state=True, values=None):
+    def stop(cls, name, state=True, values=None, value=None, mllog_key=None, suppress_stopwatch=False, suppress_mllog=False,
+             metadata=None):
         """
         stops the timer with a given name.
 
         :param name: the name of the timer
         :type name: string
+        :param state: When true, updates the status of the timer.
+        :type state: bool
+        :param mllog_key: Specifies the string name of an mllog constant to
+                          associate to this timer start.  If no value is passed
+                          and mllogging is enabled, then `name` is used.
+        :type mllog_key: string
+        :param suppress_stopwatch: When true, prevents all traditional
+                                   stopwatch logic from running.  This is
+                                   useful when attempting to interact with
+                                   mllog-only.
+        :type suppress_stopwatch: bool
+        :param suppress_mllog: When true, prevents all mllog events from
+                               executing.  Useful when working with stopwatch
+                               timers-only.
+
+        :returns: None
+        :rtype: None
         """
-        cls.timer_end[name] = time.time()
-        # if cumulate:
-        #    cls.timer_end[name] = cls.timer_end[name] + cls.timer_last[name]
-        cls.timer_sum[name] = cls.timer_sum[name] + cls.timer_end[name] - cls.timer_start[name]
-        cls.timer_status[name] = state
-        if values:
-            StopWatch.timer_values[name] = values
+        values = values or value
 
-        if cls.mllogging:
-            if values is not None:
-                cls.mllogger.end(key=f"mllog-timer-{name}", value=str(values))
+        if not suppress_stopwatch:
+            cls.timer_end[name] = time.time()
+            # if cumulate:
+            #    cls.timer_end[name] = cls.timer_end[name] + cls.timer_last[name]
+            cls.timer_sum[name] = cls.timer_sum[name] + cls.timer_end[name] - cls.timer_start[name]
+            cls.timer_status[name] = state
+            if values:
+                StopWatch.timer_values[name] = values
+
+        if cls.mllogging and not suppress_mllog:
+            if mllog_key is None:
+                key = name
             else:
-                cls.mllogger.end(key=f"mllog-timer-{name}")
+                key = cls._mllog_lookup(mllog_key)
+            if values is not None:
+                if isinstance(values, dict):
+                    values['name'] = name
+                elif isinstance(values, list):
+                    values += list(name)
+                else:
+                    values = f"Name: {name}, {values}"
 
+                cls.mllogger.end(key=key, value=str(values), metadata=metadata)
+            else:
+                cls.mllogger.end(key=key, value=name, metadata=metadata)
 
-        if cls.debug:
+        if cls.debug and not suppress_stopwatch:
             print("Timer", name, "stopped ...")
 
     @classmethod
@@ -555,6 +798,7 @@ class StopWatch(object):
                   sum=True,
                   node=None,
                   user=None,
+                  version=None,
                   attributes=None,
                   total=False,
                   filename=None):
@@ -619,7 +863,7 @@ class StopWatch(object):
                     }
                     try:
                         total_time = total_time + StopWatch.get(timer)
-                    except:
+                    except:  # noqa: E722
                         pass
                     for attribute in ["uname.node",
                                       "user",
@@ -634,6 +878,9 @@ class StopWatch(object):
                         else:
                             data_timers[timer][attribute] = data_platform[attribute]
 
+                    if version is not None:
+                        data_timers[timer]["platform.version"] = version
+
                 # print(Printer.attribute(data_timers, header=["Command", "Time/s"]))
 
                 if 'benchmark_start_stop' in data_timers:
@@ -734,7 +981,6 @@ class StopWatch(object):
         if filename:
             writefile(filename, content)
 
-
     def load(filename,
              label=["name"], label_split_char=" ",
              attributes=['timer',
@@ -787,7 +1033,6 @@ class StopWatch(object):
                 "data": data}
 
 
-
 class StopWatchBlock:
 
     def __init__(self, name, data=None, log=sys.stdout, mode="w"):
@@ -800,7 +1045,6 @@ class StopWatchBlock:
             self.is_file = True
             self.log = open(log, mode)
 
-
     def __enter__(self):
         StopWatch.start(self.name)
         return StopWatch.get(self.name)
@@ -810,9 +1054,8 @@ class StopWatchBlock:
         StopWatch.stop(self.name)
         entry = StopWatch.get(self.name)
         if self.data:
-            print (f"# {self.name}, {entry}, {self.start}, {self.stop}, {self.data}", file=self.log)
+            print(f"# {self.name}, {entry}, {self.start}, {self.stop}, {self.data}", file=self.log)
         else:
-            print (f"# {self.name}, {entry}, {self.start}, {self.stop}", file=self.log)
+            print(f"# {self.name}, {entry}, {self.start}, {self.stop}", file=self.log)
         if self.is_file:
             self.log.close()
-
